// Ionic Starter App
/* eslint-disable */
// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.services' is found in services.js
// 'starter.controllers' is found in controllers.js

(function () {
  // 严格模式

  angular.module('main', ['ionic', 'nsPopover', 'ngCordova', 'auth', 'common', 'index', 'userCenter', 'find', 'download'])

    .factory('Application', function ($window) {
      return {
        setInitialRun: function (initial) {
          $window.localStorage.initialRun = (initial ? 'true' : 'false');
        },
        isInitialRun: function () {
          var value = $window.localStorage.initialRun || 'true';
          return value == 'true';
        }
      };
    })

    /*
     * 常量配置
     */
    .constant('loginResultConstant', {
      SUCCESS: '000',
      FAIL: '001',
      PWDERR: '002',
      USERNAMEERR: '003',
      REQUESTERR: '999'
    })

    /*
     * 配置块。
     * AngularJS在模块加载时会执行这个函数。
     */
    .config(['$stateProvider', '$urlRouterProvider', '$ionicConfigProvider', '$httpProvider',
      function ($stateProvider, $urlRouterProvider, $ionicConfigProvider, $httpProvider) {
        // 在 iOS中, tabs一直处于底部.
        // 在android中, tabs一直在顶部
        // ionic中默认安装后导航在底部解决方案
        $ionicConfigProvider.platform.ios.tabs.style('standard');
        $ionicConfigProvider.platform.ios.tabs.position('bottom');
        $ionicConfigProvider.platform.android.tabs.style('standard');
        $ionicConfigProvider.platform.android.tabs.position('bottom');

        $ionicConfigProvider.platform.ios.navBar.alignTitle('center');
        $ionicConfigProvider.platform.android.navBar.alignTitle('center');

        $ionicConfigProvider.platform.ios.backButton.previousTitleText('').icon('ion-ios-arrow-thin-left');
        $ionicConfigProvider.platform.android.backButton.previousTitleText('').icon('ion-android-arrow-back');

        $ionicConfigProvider.platform.ios.views.transition('ios');
        $ionicConfigProvider.platform.android.views.transition('android');

        // 解决如下问题：
        // 1.Ajax跨域POST请求中，在主请求以前伴随另外一个OPTIONS请求的问题;
        // 2.POST表单提交问题,编码格式为UTF-8;

        /* Function: Override $http service's default transformRequest.
         *           Converts an object to x-www-form-urlencoded serialization.
         * @param {Object} obj
         * @return {String}
         */
        $httpProvider.defaults.transformRequest = function (data) {
          /* 简易写法
           var str = [];
           for(var p in data){
           str.push(encodeURIComponent(p) + "=" + encodeURIComponent(data[p]));
           }
           return str.join("&");
           */

          // console.log('Transform Request: ');

          var param = function (obj) {
            var query = '';
            var name,
              value,
              fullSubName,
              subName,
              subValue,
              innerObj,
              i;

            for (name in obj) {
              value = obj[name];

              if (value instanceof Array) {
                for (i = 0; i < value.length; ++i) {
                  subValue = value[i];
                  fullSubName = name + '[' + i +']';
                  innerObj = {};
                  innerObj[fullSubName] = subValue;
                  query += param(innerObj) + '&';
                }
              } else if (value instanceof Object) {
                for (subName in value) {
                  subValue = value[subName];
                  fullSubName = name + '[' + subName + ']';
                  innerObj = {};
                  innerObj[fullSubName] = subValue;
                  query += param(innerObj) + '&';
                }
              } else if (value !== undefined && value !== null) {
                query += encodeURIComponent(name) + '=' + encodeURIComponent(value) + '&';
              }
            }
            return query.length ? query.substr(0, query.length - 1) : query;
          };
          return angular.isObject(data) && String(data) !== '[object File]' ? param(data) : data;
        };

        $httpProvider.defaults.headers.post = {
          'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8'
        };
      }
    ])

    /*
     * 运行块。
     * AngularJS在注入器创建后会执行这个函数。
     */
    .run(['$rootScope', '$ionicPlatform', '$state', '$location', '$window', '$ionicPopup',
      '$ionicHistory', '$ionicLoading', '$timeout', 'authService', 'downloadService', 'Application', 'AppUpdateService',
      function (
        $rootScope, $ionicPlatform, $state, $location, $window, $ionicPopup,
        $ionicHistory, $ionicLoading, $timeout, authService, downloadService, Application, AppUpdateService
      ) {
        /*
         *
         */

        $ionicPlatform.ready(function () {
          // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
          // for form inputs)
          if (window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard) {
            cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
          }

          if (window.StatusBar) {
            // org.apache.cordova.statusbar required
            StatusBar.styleDefault();
          }

          setTimeout(function () {
            navigator.splashscreen.hide();
          }, 100);

          document.addEventListener('deviceready', function () {
            document.addEventListener('offline', onOffline, false);

            function onOffline(e) {
              // Handle the offline event

              $ionicLoading.show({
                template: '网络未连接，请稍后重试！'
              });

              $timeout(function () {
                $ionicLoading.hide();
                ionic.Platform.exitApp();
              }, 3000);
            }
          }, false);

          AppUpdateService.checkVersion();
          // 进入APP时判断进入哪个页面
          // var state = "app.index.home";  // whatever, the main page of your app
          // //第一次打开
          // if (Application.isInitialRun()) {
          //     Application.setInitialRun(false);
          //     state = "prologue";
          // }
          // $state.go(state);
        });

        /*
         *主页面显示退出提示框
         */
        $ionicPlatform.registerBackButtonAction(function (e) {
          e.preventDefault();

          function showConfirm() {
            var confirmPopup = $ionicPopup.confirm({
              title: '<div align="left"><i class="icon ion-android-alert  clam;text-align:center;"><strong>&nbsp;温馨提示：</strong></i></div>',
              template: '您确定要退出应用吗?',
              cancelText: '<span color="#FFFFFF">取消</span>',
              okText: '<span color="#FFFFFF">确认</span>',
              okType: 'button-clam',
            });
            confirmPopup.then(function (res) {
              if (res) {
                ionic.Platform.exitApp();
              } else {}
            });
          }

          // Is there a page to go back to?
          if ($location.path() == '/app/index/home' || $location.path() == '/app/userCenter/home' || $location.path() == '/app/download/home' || $location.path() == '/app/find/home' || $location.path() == '/') {
            showConfirm();
          } else if ($ionicHistory.backView()) {
            // Go back in history
            window.history.back();
          } else {
            // This is the last page: Show confirmation popup
            showConfirm();
          }

          return false;
        }, 101);

        $rootScope.$on('$stateChangeStart', function (event, toState, toParams, fromState, fromParams) {

          var isFilter = authService.isFilterState(toState.name);

          if (!isFilter) {
            if (window.localStorage.isLogin === undefined || !window.localStorage.isLogin) {
              if (toState.name == 'app.login') {
                console.log('main: The state is app.login, so go directly to login page!');
              } else {
                console.log('main: The user has not logged in, so go to login page!');

                var nextURL = authService.findUrlByState(toState.name);

                // 参数替换
                for (var name in toParams) {
                  var paramName = ':' + name;

                  if (nextURL.indexOf(paramName) != -1) {
                    nextURL = nextURL.replace(paramName, toParams[name]);
                  }
                }
                /**
                 * $location存在页面不跳转的情况，原因不明。
                 * 暂时不用。
                 */
                // $location.path('/app/login/' + encodeURIComponent(authURL));
                // $window.location.href = '#/app/login?nextURL=' + encodeURIComponent(nextURL);

                /**
                 * Modal模式。
                 * 支持$window.location.href、$state.go、$location.path状态或页面跳转方式。
                 * 在正式环境使用。
                 * 此处屏蔽，主要是屏蔽用户登录，便于演示。
                 */
                /**/
                // 未登录，阻止页面跳转。
                event.preventDefault();
                // 弹出登录页面
                $state.go('app.login');
              }
            } else {
              console.log('main: The user has been logged in, so skip app.login!');
            }
          } else {
            console.log('main: The state is in the filterURLs, so is out of authentication, go directly to this state:' + toState.name);
          }
        });


        /*
         * 状态未发现事件
         */
        $rootScope.$on('$stateNotFound', function (event, unfoundState, fromState, fromParams) {

        });
      }
    ]);
}());
